@@ -35,9 +35,10 @@ from adi.context_manager import context_manager
 from adi.jesd import jesd
 from adi.obs import obs
 from adi.rx_tx import rx_tx
+from adi.sync_start import sync_start
 
 
-class ad9371(rx_tx, context_manager):
+class ad9371(rx_tx, context_manager, sync_start):
     """ AD9371 Transceiver """
 
     _complex_data = True
@@ -46,15 +47,25 @@ class ad9371(rx_tx, context_manager):
     _obs_channel_names = ["voltage0_i", "voltage0_q"]
     _device_name = ""
 
-    def __init__(self, uri="", username="root", password="analog"):
-
+    def __init__(
+        self, uri="", username="root", password="analog", disable_jesd_control=False
+    ):
+        """Initialize AD9371 interface class.
+        Args:
+            uri (str): URI of target platform with AD9371
+            username (str): SSH username for target board. Required for JESD monitoring
+            password (str): SSH password for target board. Required for JESD monitoring
+            disable_jesd_control (bool): Disable JESD status monitoring over SSH
+        """
         context_manager.__init__(self, uri, self._device_name)
 
         self._ctrl = self._ctx.find_device("ad9371-phy")
         self._rxadc = self._ctx.find_device("axi-ad9371-rx-hpc")
         self._rxobs = self._ctx.find_device("axi-ad9371-rx-obs-hpc")
         self._txdac = self._ctx.find_device("axi-ad9371-tx-hpc")
-        self._jesd = jesd(uri, username=username, password=password)
+
+        if not disable_jesd_control and jesd:
+            self._jesd = jesd(uri, username=username, password=password)
 
         rx_tx.__init__(self)
 
